#
# author: Jungtaek Kim (jungtaek.kim.mail@gmail.com)
# last updated: January 4, 2023
#

import numpy as np

import bayeso_benchmarks as bb


def get_benchmark(str_fun, seed=None, **kwargs):
    count = 0

    for class_benchmark in bb.all_benchmarks:
        if str_fun == class_benchmark.__name__.lower():
            target_class = class_benchmark
            count += 1

    if count == 0:
        raise ValueError('missing str_fun.')
    elif count > 1:
        raise ValueError('duplicate class name.')

    if str_fun in [
        'ackley',
        'cosines',
        'griewank',
        'levy',
        'rastrigin',
        'rosenbrock',
        'sphere',
        'zakharov',
    ]:
        assert 'dim' in kwargs
        dim = kwargs['dim']

        benchmark = target_class(dim, seed=seed)
    elif str_fun == 'constant':
        assert 'bounds' in kwargs
        assert 'constant' in kwargs
        bounds = kwargs['bounds']
        constant = kwargs['constant']

        benchmark = target_class(bounds=bounds, constant=constant, seed=seed)
    elif str_fun == 'linear':
        assert 'bounds' in kwargs
        assert 'slope' in kwargs
        bounds = kwargs['bounds']
        slope = kwargs['slope']

        benchmark = target_class(bounds=bounds, slope=slope, seed=seed)
    elif str_fun == 'step':
        assert 'steps' in kwargs
        assert 'step_values' in kwargs
        steps = kwargs['steps']
        step_values = kwargs['step_values']

        benchmark = target_class(steps=steps, step_values=step_values, seed=seed)
    else:
        benchmark = target_class(seed=seed)

    return benchmark

def pdf_two_dim_normal(bx, mu, Cov):
    assert bx.shape[0] == mu.shape[0] == Cov.shape[0] == Cov.shape[1] == 2
    
    dim = bx.shape[0]

    term_first = ((2.0 * np.pi)**(-0.5 * dim)) * (np.linalg.det(Cov)**(-0.5))
    term_second = np.exp(-0.5 * np.dot(np.dot((bx - mu), np.linalg.inv(Cov)), bx - mu))
    
    return term_first * term_second



#
# author: Jungtaek Kim (jungtaek.kim.mail@gmail.com)
# last updated: August 4, 2023
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx, a, b, c, r, s, t):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx
    assert isinstance(a, float)
    assert isinstance(b, float)
    assert isinstance(c, float)
    assert isinstance(r, float)
    assert isinstance(s, float)
    assert isinstance(t, float)

    y = a * (bx[1] - b * bx[0]**2 + c * bx[0] - r)**2 + s * (1 - t) * np.cos(bx[0]) + s
    return y


class Branin(Function):
    def __init__(self,
        a=1.0,
        b=5.1 / (4.0 * np.pi**2),
        c=5 / np.pi,
        r=6.0,
        s=10.0,
        t=1 / (8 * np.pi),
        seed=None
    ):
        assert isinstance(a, float)
        assert isinstance(b, float)
        assert isinstance(c, float)
        assert isinstance(r, float)
        assert isinstance(s, float)
        assert isinstance(t, float)
        assert isinstance(seed, (type(None), int))

        dim_bx = 2
        bounds = np.array([
            [-5, 10],
            [0, 15],
        ])
        assert bounds.shape[0] == dim_bx
        assert bounds.shape[1] == 2

        global_minimizers = np.array([
            [-np.pi, 12.275],
            [np.pi, 2.275],
            [9.42478, 2.475],
        ])
        global_minimum = 0.3978873577
        function = lambda bx: fun_target(bx, dim_bx, a, b, c, r, s, t)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jungtaek.kim.mail@gmail.com)
# last updated: August 4, 2023
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx

    alpha = np.array([1.0, 1.2, 3.0, 3.2])
    A = np.array([
        [3.0, 10.0, 30.0],
        [0.1, 10.0, 35.0],
        [3.0, 10.0, 30.0],
        [0.1, 10.0, 35.0],
    ])
    P = 1e-4 * np.array([
        [3689, 1170, 2673],
        [4699, 4387, 7470],
        [1091, 8732, 5547],
        [381, 5743, 8828],
    ])

    outer = 0.0
    for i_ in range(0, 4):
        inner = 0.0
        for j_ in range(0, 3):
            inner += A[i_, j_] * (bx[j_] - P[i_, j_])**2
        outer += alpha[i_] * np.exp(-1.0 * inner)

    y = -1.0 * outer
    return y


class Hartmann3D(Function):
    def __init__(self, seed=None):
        assert isinstance(seed, (type(None), int))

        dim_bx = 3
        bounds = np.array([
            [0.0, 1.0],
            [0.0, 1.0],
            [0.0, 1.0],
        ])
        assert bounds.shape[0] == dim_bx
        assert bounds.shape[1] == 2

        global_minimizers = np.array([
            [0.11458889, 0.55564889, 0.85254698],
        ])
        global_minimum = -3.8627797874
        function = lambda bx: fun_target(bx, dim_bx)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: February 8, 2021
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx,
    a=20.0,
    b=0.2,
    c=2.0*np.pi
):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx
    assert isinstance(a, float)
    assert isinstance(b, float)
    assert isinstance(c, float)

    y = -a * np.exp(-b * np.linalg.norm(bx, ord=2, axis=0) * np.sqrt(1.0 / dim_bx)) - np.exp(1.0 / dim_bx * np.sum(np.cos(c * bx), axis=0)) + a + np.exp(1.0)
    return y


class Ackley(Function):
    def __init__(self, dim_problem, seed=None):
        assert isinstance(dim_problem, int)
        assert isinstance(seed, (type(None), int))

        dim_bx = np.inf
        bounds = np.array([
            [-32.768, 32.768],
        ])
        global_minimizers = np.array([
            [0.0],
        ])
        global_minimum = 0.0
        dim_problem = dim_problem

        function = lambda bx: fun_target(bx, dim_problem)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, dim_problem=dim_problem, seed=seed)



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: January 3, 2023
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx

    y = -1.0 * np.cos(bx[0]) * np.cos(bx[1]) * np.exp(-1.0 * (bx[0] - np.pi)**2 - (bx[1] - np.pi)**2)
    return y


class Easom(Function):
    def __init__(self, seed=None):
        assert isinstance(seed, (type(None), int))

        dim_bx = 2
        bounds = np.array([
            [-100, 100],
            [-100, 100],
        ])
        assert bounds.shape[0] == dim_bx
        assert bounds.shape[1] == 2

        global_minimizers = np.array([
            [np.pi, np.pi],
        ])
        global_minimum = -1.0
        function = lambda bx: fun_target(bx, dim_bx)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: December 4, 2022
#

import numpy as np
import pytest

from bayeso_benchmarks.two_dim_branin import *

class_fun = Branin
str_name = 'branin'

TEST_EPSILON = 1e-5
SCALE_NOISE = 2.0
SEED = 42


def test_init():
    obj_fun = class_fun()

    with pytest.raises(AssertionError) as error:
        class_fun(a='abc')
    with pytest.raises(AssertionError) as error:
        class_fun(b='abc')
    with pytest.raises(AssertionError) as error:
        class_fun(c='abc')
    with pytest.raises(AssertionError) as error:
        class_fun(r='abc')
    with pytest.raises(AssertionError) as error:
        class_fun(s='abc')
    with pytest.raises(AssertionError) as error:
        class_fun(t='abc')
    with pytest.raises(AssertionError) as error:
        class_fun(seed=1.0)
    with pytest.raises(AssertionError) as error:
        class_fun(seed='abc')

def test_validate_properties():
    obj_fun = class_fun()
    obj_fun.validate_properties()

def test_output():
    obj_fun = class_fun()
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [308.12909601],
        [10.30790849],
        [10.96088904],
        [106.56869776],
        [24.12996441],
        [22.16653996],
        [17.50829952],
        [150.45202034],
        [145.87219088],
    ])

    print(grids)
    print(obj_fun.output(grids))
    print(np.abs(obj_fun.output(grids) - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(obj_fun.output(grids) - truths_grids) < TEST_EPSILON)

def test_call():
    obj_fun = class_fun()
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [308.12909601],
        [10.30790849],
        [10.96088904],
        [106.56869776],
        [24.12996441],
        [22.16653996],
        [17.50829952],
        [150.45202034],
        [145.87219088],
    ])

    print(grids)
    print(obj_fun(grids))
    print(np.abs(obj_fun(grids) - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(obj_fun(grids) - truths_grids) < TEST_EPSILON)

def test_output_constant_noise():
    obj_fun = class_fun()
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [308.12909601],
        [10.30790849],
        [10.96088904],
        [106.56869776],
        [24.12996441],
        [22.16653996],
        [17.50829952],
        [150.45202034],
        [145.87219088],
    ])

    print(grids)
    print(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE))
    print(np.abs(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE) - truths_grids) < TEST_EPSILON + SCALE_NOISE)
    assert np.all(np.abs(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE) - truths_grids) < TEST_EPSILON + SCALE_NOISE)

def test_output_gaussian_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [309.12252432],
        [10.03137988],
        [12.25626611],
        [109.61475748],
        [23.66165766],
        [21.69826604],
        [20.66672515],
        [151.9868898],
        [144.93324211],
    ])
    outputs = obj_fun.output_gaussian_noise(grids, scale_noise=SCALE_NOISE)

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_output_sparse_gaussian_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [309.12252432],
        [10.03137988],
        [10.96088904],
        [106.56869776],
        [24.12996441],
        [21.69826604],
        [17.50829952],
        [151.9868898],
        [144.93324211],
    ])
    outputs = obj_fun.output_sparse_gaussian_noise(grids, scale_noise=SCALE_NOISE, sparsity=0.3)

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_output_student_t_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [309.27833061],
        [8.14452801],
        [13.59900607],
        [104.98514624],
        [25.60278992],
        [25.95324732],
        [13.58000707],
        [149.34002319],
        [144.72897096],
    ])
    outputs = obj_fun.output_student_t_noise(grids, scale_noise=SCALE_NOISE, dof=4.0)

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_output_sparse_student_t_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [309.27833061],
        [8.14452801],
        [10.96088904],
        [106.56869776],
        [24.12996441],
        [22.16653996],
        [13.58000707],
        [150.45202034],
        [145.87219088],
    ])
    outputs = obj_fun.output_sparse_student_t_noise(grids, scale_noise=SCALE_NOISE, dof=4.0, sparsity=0.3)

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_name():
    obj_fun = class_fun()
    assert obj_fun.name == str_name

    assert obj_fun.__class__.__name__.lower() == str_name
    assert obj_fun.__class__.__qualname__.lower() == str_name



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: February 8, 2021
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx

    y = (1.5 - bx[0] + bx[0] * bx[1])**2 + (2.25 - bx[0] + bx[0] * bx[1]**2)**2 + (2.625 - bx[0] + bx[0] * bx[1]**3)**2
    return y


class Beale(Function):
    def __init__(self, seed=None):
        assert isinstance(seed, (type(None), int))

        dim_bx = 2
        bounds = np.array([
            [-4.5, 4.5],
            [-4.5, 4.5],
        ])
        assert bounds.shape[0] == dim_bx
        assert bounds.shape[1] == 2

        global_minimizers = np.array([
            [3.0, 0.5],
        ])
        global_minimum = 0.0
        function = lambda bx: fun_target(bx, dim_bx)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jungtaek.kim.mail@gmail.com)
# last updated: August 4, 2023
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx

    y = np.sin(10.0 * np.pi * bx[0]) / (2 * bx[0]) + (bx[0] - 1.0)**4
    return y


class GramacyAndLee2012(Function):
    def __init__(self, seed=None):
        assert isinstance(seed, (type(None), int))

        dim_bx = 1
        bounds = np.array([
            [0.5, 2.5],
        ])
        assert bounds.shape[0] == dim_bx
        assert bounds.shape[1] == 2

        global_minimizers = np.array([
            [0.54856344],
        ])
        global_minimum = -0.8690111350
        function = lambda bx: fun_target(bx, dim_bx)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: January 6, 2023
#

import numpy as np
import pytest

from bayeso_benchmarks import utils

TEST_EPSILON = 1e-5


def test_get_benchmark():
    with pytest.raises(TypeError) as error:
        benchmark = utils.get_benchmark()
    with pytest.raises(ValueError) as error:
        benchmark = utils.get_benchmark('abc', seed=None)

    with pytest.raises(AssertionError) as error:
        benchmark = utils.get_benchmark('ackley')
    with pytest.raises(AssertionError) as error:
        benchmark = utils.get_benchmark('ackley', seed='abc')

    benchmark = utils.get_benchmark('ackley', dim=4, seed=42)
    print(benchmark.output(np.array([0.0, 0.0, 0.0, 0.0])))

    with pytest.raises(AssertionError) as error:
        benchmark = utils.get_benchmark('cosines')

    benchmark = utils.get_benchmark('cosines', dim=4, seed=None)

    with pytest.raises(AssertionError) as error:
        benchmark = utils.get_benchmark('griewank')

    benchmark = utils.get_benchmark('griewank', dim=4, seed=None)

    with pytest.raises(AssertionError) as error:
        benchmark = utils.get_benchmark('levy')

    benchmark = utils.get_benchmark('levy', dim=2, seed=None)

    with pytest.raises(AssertionError) as error:
        benchmark = utils.get_benchmark('rastrigin')

    benchmark = utils.get_benchmark('rastrigin', dim=8, seed=None)

    with pytest.raises(AssertionError) as error:
        benchmark = utils.get_benchmark('rosenbrock')

    benchmark = utils.get_benchmark('rosenbrock', dim=8, seed=None)

    with pytest.raises(AssertionError) as error:
        benchmark = utils.get_benchmark('sphere')

    benchmark = utils.get_benchmark('sphere', dim=16, seed=None)

    with pytest.raises(AssertionError) as error:
        benchmark = utils.get_benchmark('zakharov')

    benchmark = utils.get_benchmark('zakharov', dim=16, seed=None)

    with pytest.raises(AssertionError) as error:
        benchmark = utils.get_benchmark('constant')
    with pytest.raises(AssertionError) as error:
        benchmark = utils.get_benchmark('constant', constant=None)
    with pytest.raises(AssertionError) as error:
        benchmark = utils.get_benchmark('constant', bounds=None)
    with pytest.raises(AssertionError) as error:
        benchmark = utils.get_benchmark('constant', bounds=np.array([0.0, 10.0]), constant=10.0, seed=None)

    benchmark = utils.get_benchmark('constant', bounds=np.array([[0.0, 10.0]]), constant=10.0, seed=None)

    benchmark = utils.get_benchmark('gramacyandlee2012')

    with pytest.raises(AssertionError) as error:
        benchmark = utils.get_benchmark('linear')

    benchmark = utils.get_benchmark('linear', bounds=np.array([[0.0, 10.0]]), slope=-1.2, seed=None)

    with pytest.raises(AssertionError) as error:
        benchmark = utils.get_benchmark('step')

    benchmark = utils.get_benchmark('step', steps=[0.0, 3.0, 7.0, 10.0], step_values=[-2.1, 4.0, 10.0], seed=None)

    benchmark = utils.get_benchmark('beale')
    benchmark = utils.get_benchmark('bohachevsky')

    benchmark = utils.get_benchmark('branin')
    print(benchmark.output(np.array([1.0, 1.0])))

    benchmark = utils.get_benchmark('bukin6')
    benchmark = utils.get_benchmark('dejong5')
    benchmark = utils.get_benchmark('dropwave')
    benchmark = utils.get_benchmark('easom')
    benchmark = utils.get_benchmark('eggholder')
    benchmark = utils.get_benchmark('goldsteinprice')
    benchmark = utils.get_benchmark('holdertable')
    benchmark = utils.get_benchmark('kim1')
    benchmark = utils.get_benchmark('kim2')
    benchmark = utils.get_benchmark('kim3')
    benchmark = utils.get_benchmark('michalewicz')
    benchmark = utils.get_benchmark('shubert')
    benchmark = utils.get_benchmark('sixhumpcamel')
    benchmark = utils.get_benchmark('threehumpcamel')

    benchmark = utils.get_benchmark('colville')
    benchmark = utils.get_benchmark('hartmann3d')
    benchmark = utils.get_benchmark('hartmann6d')

def test_pdf_two_dim_normal():
    bx = np.array([0.0, 1.0])
    mu = np.array([1.0, 1.0])
    Cov = np.array([
        [2.0, 1.0],
        [1.0, 2.0],
    ])

    with pytest.raises(AssertionError) as error:
        value = utils.pdf_two_dim_normal(np.array([1.0, 1.0, 1.0]), mu, Cov)
    with pytest.raises(AssertionError) as error:
        value = utils.pdf_two_dim_normal(np.array([2.0]), mu, Cov)
    with pytest.raises(AssertionError) as error:
        value = utils.pdf_two_dim_normal(bx, np.array([1.0, 1.0, 1.0]), Cov)
    with pytest.raises(AssertionError) as error:
        value = utils.pdf_two_dim_normal(bx, np.array([3.0]), Cov)

    value = utils.pdf_two_dim_normal(bx, mu, Cov)
    print(value)

    assert np.abs(0.06584073599896273 - value) < TEST_EPSILON



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: January 3, 2023
#

import numpy as np
import pytest

from bayeso_benchmarks.two_dim_shubert import *

class_fun = Shubert
str_name = 'shubert'

TEST_EPSILON = 1e-5
SCALE_NOISE = 2.0
SEED = 42


def test_init():
    obj_fun = class_fun()

    with pytest.raises(AssertionError) as error:
        class_fun(seed=1.0)
    with pytest.raises(AssertionError) as error:
        class_fun(seed='abc')

def test_validate_properties():
    obj_fun = class_fun()
    obj_fun.validate_properties()

def test_output():
    obj_fun = class_fun()
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [0.06674108],
        [1.15175294],
        [0.86375707],
        [1.15175294],
        [19.87583625],
        [14.90588261],
        [0.86375707],
        [14.90588261],
        [11.17866608],
    ])

    print(grids)
    print(obj_fun.output(grids))
    print(np.abs(obj_fun.output(grids) - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(obj_fun.output(grids) - truths_grids) < TEST_EPSILON)

def test_call():
    obj_fun = class_fun()
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [0.06674108],
        [1.15175294],
        [0.86375707],
        [1.15175294],
        [19.87583625],
        [14.90588261],
        [0.86375707],
        [14.90588261],
        [11.17866608],
    ])

    print(grids)
    print(obj_fun(grids))
    print(np.abs(obj_fun(grids) - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(obj_fun(grids) - truths_grids) < TEST_EPSILON)

def test_output_constant_noise():
    obj_fun = class_fun()
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [2.06674108],
        [3.15175294],
        [2.86375707],
        [3.15175294],
        [21.87583625],
        [16.90588261],
        [2.86375707],
        [16.90588261],
        [13.17866608],
    ])

    print(grids)
    print(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE))
    print(np.abs(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE) - truths_grids) < TEST_EPSILON + SCALE_NOISE)
    assert np.all(np.abs(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE) - truths_grids) < TEST_EPSILON + SCALE_NOISE)

def test_output_gaussian_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [1.06016939],
        [0.87522434],
        [2.15913415],
        [4.19781266],
        [19.4075295],
        [14.4376087],
        [4.02218271],
        [16.44075207],
        [10.2397173],
    ])
    outputs = obj_fun.output_gaussian_noise(grids, scale_noise=SCALE_NOISE)

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_output_sparse_gaussian_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [1.06016939],
        [0.87522434],
        [0.86375707],
        [1.15175294],
        [19.87583625],
        [14.4376087],
        [0.86375707],
        [16.44075207],
        [10.2397173],
    ])
    outputs = obj_fun.output_sparse_gaussian_noise(grids, scale_noise=SCALE_NOISE, sparsity=0.3)

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_output_student_t_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [1.21597568],
        [-1.01162753],
        [3.50187411],
        [-0.43179858],
        [21.34866175],
        [18.69258998],
        [-3.06453537],
        [13.79388546],
        [10.03544615],
    ])
    outputs = obj_fun.output_student_t_noise(grids, scale_noise=SCALE_NOISE, dof=4.0)

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_output_sparse_student_t_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [1.21597568],
        [-1.01162753],
        [0.86375707],
        [1.15175294],
        [19.87583625],
        [14.90588261],
        [-3.06453537],
        [14.90588261],
        [11.17866608],
    ])
    outputs = obj_fun.output_sparse_student_t_noise(grids, scale_noise=SCALE_NOISE, dof=4.0, sparsity=0.3)

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_name():
    obj_fun = class_fun()
    assert obj_fun.name == str_name

    assert obj_fun.__class__.__name__.lower() == str_name
    assert obj_fun.__class__.__qualname__.lower() == str_name



#
# author: Jungtaek Kim (jungtaek.kim.mail@gmail.com)
# last updated: August 4, 2023
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx

    y = np.sin(bx[0]) + np.cos(bx[1]) + 0.016 * (bx[0] - 5.0)**2 + 0.008 * (bx[1] - 5.0)**2
    return y


class Kim1(Function):
    def __init__(self, seed=None):
        assert isinstance(seed, (type(None), int))

        dim_bx = 2
        bounds = np.array([
            [-16.0, 16.0],
            [-16.0, 16.0],
        ])
        assert bounds.shape[0] == dim_bx
        assert bounds.shape[1] == 2

        global_minimizers = np.array([
            [4.72130726, 3.17086303],
        ])
        global_minimum = -1.9715232348
        function = lambda bx: fun_target(bx, dim_bx)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jungtaek.kim.mail@gmail.com)
# last updated: August 4, 2023
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx

    y = (4.0 - 2.1 * bx[0]**2 + bx[0]**4 / 3.0) * bx[0]**2 + bx[0] * bx[1] + (-4.0 + 4.0 * bx[1]**2) * bx[1]**2
    return y


class SixHumpCamel(Function):
    def __init__(self, seed=None):
        assert isinstance(seed, (type(None), int))

        dim_bx = 2
        bounds = np.array([
            [-3.0, 3.0],
            [-2.0, 2.0],
        ])
        assert bounds.shape[0] == dim_bx
        assert bounds.shape[1] == 2

        global_minimizers = np.array([
            [0.08984201, -0.7126564],
            [-0.08984201, 0.7126564],
        ])
        global_minimum = -1.0316284535
        function = lambda bx: fun_target(bx, dim_bx)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: February 8, 2021
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx, steps, step_values):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx
    assert isinstance(steps, list)
    assert isinstance(step_values, list)
    assert len(steps) == len(step_values) + 1
    assert isinstance(steps[0], float)
    assert isinstance(step_values[0], float)

    y = None
    for ind_step in range(0, len(steps) - 1):
        if ind_step < (len(steps) - 2) and steps[ind_step] <= bx[0] and bx[0] < steps[ind_step+1]:
            y = step_values[ind_step]
            break
        elif ind_step == (len(steps) - 2) and steps[ind_step] <= bx[0] and bx[0] <= steps[ind_step+1]:
            y = step_values[ind_step]
            break

    if y is None:
        raise ValueError('Conditions for steps')
    return y


class Step(Function):
    def __init__(self,
        steps=[-10., -5., 0., 5., 10.],
        step_values=[-2., 0., 1., -1.],
        seed=None
    ):
        assert isinstance(steps, list)
        assert isinstance(step_values, list)
        assert isinstance(seed, (type(None), int))
        assert len(steps) == len(step_values) + 1
        assert isinstance(steps[0], float)
        assert isinstance(step_values[0], float)
        assert np.all(np.sort(steps) == np.asarray(steps))

        dim_bx = 1
        bounds = np.array([
            [np.min(steps), np.max(steps)],
        ])
        assert bounds.shape[0] == dim_bx
        assert bounds.shape[1] == 2

        global_minimizers = np.array([
            [steps[np.argmin(step_values)]],
        ])
        global_minimum = np.min(step_values)
        function = lambda bx: fun_target(bx, dim_bx, steps, step_values)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: January 6, 2023
#

import numpy as np
import pytest

from bayeso_benchmarks.two_dim_bukin6 import *

class_fun = Bukin6
str_name = 'bukin6'

TEST_EPSILON = 1e-5
SCALE_NOISE = 2.0
SEED = 42


def test_init():
    obj_fun = class_fun()

    with pytest.raises(AssertionError) as error:
        class_fun(seed=1.0)
    with pytest.raises(AssertionError) as error:
        class_fun(seed='abc')

def test_validate_properties():
    obj_fun = class_fun()
    obj_fun.validate_properties()

def test_output():
    obj_fun = class_fun()
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [229.17878475],
        [200.0],
        [180.32756377],
        [150.05],
        [100.0],
        [50.05],
        [86.65254038],
        [141.42135624],
        [165.88123952],
    ])

    print(grids)
    print(obj_fun.output(grids))
    print(np.abs(obj_fun.output(grids) - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(obj_fun.output(grids) - truths_grids) < TEST_EPSILON)

def test_call():
    obj_fun = class_fun()
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [229.17878475],
        [200.0],
        [180.32756377],
        [150.05],
        [100.0],
        [50.05],
        [86.65254038],
        [141.42135624],
        [165.88123952],
    ])

    print(grids)
    print(obj_fun(grids))
    print(np.abs(obj_fun(grids) - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(obj_fun(grids) - truths_grids) < TEST_EPSILON)

def test_output_constant_noise():
    obj_fun = class_fun()
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [231.17878475],
        [202.0],
        [182.32756377],
        [152.05],
        [102.0],
        [52.05],
        [88.65254038],
        [143.42135624],
        [167.88123952],
    ])

    print(grids)
    print(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE))
    print(np.abs(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE) - truths_grids) < TEST_EPSILON + SCALE_NOISE)
    assert np.all(np.abs(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE) - truths_grids) < TEST_EPSILON + SCALE_NOISE)

def test_output_gaussian_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [230.17221305],
        [199.7234714],
        [181.62294085],
        [153.09605971],
        [99.53169325],
        [49.58172609],
        [89.81096601],
        [142.9562257],
        [164.94229075],
    ])
    outputs = obj_fun.output_gaussian_noise(grids, scale_noise=SCALE_NOISE)

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_output_sparse_gaussian_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [230.17221305],
        [199.7234714],
        [180.32756377],
        [150.05],
        [100.0],
        [49.58172609],
        [86.65254038],
        [142.9562257 ],
        [164.94229075],
    ])
    outputs = obj_fun.output_sparse_gaussian_noise(grids, scale_noise=SCALE_NOISE, sparsity=0.3)

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_output_student_t_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [230.32801935],
        [197.83661953],
        [182.96568081],
        [148.46644847],
        [101.4728255],
        [53.83670737],
        [82.72424794],
        [140.30935909],
        [164.7380196],
    ])
    outputs = obj_fun.output_student_t_noise(grids, scale_noise=SCALE_NOISE, dof=4.0)

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_output_sparse_student_t_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [230.32801935],
        [197.83661953],
        [180.32756377],
        [150.05],
        [100.0],
        [50.05],
        [82.72424794],
        [141.42135624],
        [165.88123952],
    ])
    outputs = obj_fun.output_sparse_student_t_noise(grids, scale_noise=SCALE_NOISE, dof=4.0, sparsity=0.3)

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_name():
    obj_fun = class_fun()
    assert obj_fun.name == str_name

    assert obj_fun.__class__.__name__.lower() == str_name
    assert obj_fun.__class__.__qualname__.lower() == str_name



#
# author: Jungtaek Kim (jungtaek.kim.mail@gmail.com)
# last updated: January 27, 2023
#


__version__ = '0.2.0'


from bayeso_benchmarks.inf_dim_ackley import Ackley
from bayeso_benchmarks.inf_dim_cosines import Cosines
from bayeso_benchmarks.inf_dim_griewank import Griewank
from bayeso_benchmarks.inf_dim_levy import Levy
from bayeso_benchmarks.inf_dim_rastrigin import Rastrigin
from bayeso_benchmarks.inf_dim_rosenbrock import Rosenbrock
from bayeso_benchmarks.inf_dim_sphere import Sphere
from bayeso_benchmarks.inf_dim_zakharov import Zakharov

from bayeso_benchmarks.one_dim_constant import Constant
from bayeso_benchmarks.one_dim_gramacyandlee2012 import GramacyAndLee2012
from bayeso_benchmarks.one_dim_linear import Linear
from bayeso_benchmarks.one_dim_step import Step

from bayeso_benchmarks.two_dim_beale import Beale
from bayeso_benchmarks.two_dim_bohachevsky import Bohachevsky
from bayeso_benchmarks.two_dim_branin import Branin
from bayeso_benchmarks.two_dim_bukin6 import Bukin6
from bayeso_benchmarks.two_dim_dejong5 import DeJong5
from bayeso_benchmarks.two_dim_dropwave import DropWave
from bayeso_benchmarks.two_dim_easom import Easom
from bayeso_benchmarks.two_dim_eggholder import Eggholder
from bayeso_benchmarks.two_dim_goldsteinprice import GoldsteinPrice
from bayeso_benchmarks.two_dim_holdertable import HolderTable
from bayeso_benchmarks.two_dim_kim1 import Kim1
from bayeso_benchmarks.two_dim_kim2 import Kim2
from bayeso_benchmarks.two_dim_kim3 import Kim3
from bayeso_benchmarks.two_dim_michalewicz import Michalewicz
from bayeso_benchmarks.two_dim_shubert import Shubert
from bayeso_benchmarks.two_dim_sixhumpcamel import SixHumpCamel
from bayeso_benchmarks.two_dim_threehumpcamel import ThreeHumpCamel

from bayeso_benchmarks.four_dim_colville import Colville
from bayeso_benchmarks.three_dim_hartmann3d import Hartmann3D
from bayeso_benchmarks.six_dim_hartmann6d import Hartmann6D


all_benchmarks = [
    Ackley,
    Cosines,
    Griewank,
    Levy,
    Rastrigin,
    Rosenbrock,
    Sphere,
    Zakharov,
    Constant,
    GramacyAndLee2012,
    Linear,
    Step,
    Beale,
    Bohachevsky,
    Branin,
    Bukin6,
    DeJong5,
    DropWave,
    Easom,
    Eggholder,
    GoldsteinPrice,
    HolderTable,
    Kim1,
    Kim2,
    Kim3,
    Michalewicz,
    Shubert,
    SixHumpCamel,
    ThreeHumpCamel,
    Colville,
    Hartmann3D,
    Hartmann6D,
]
num_benchmarks = len(all_benchmarks)



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: February 8, 2021
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx

    y = 0.0

    for ind in range(0, dim_bx - 1):
        y += 100 * (bx[ind+1] - bx[ind]**2)**2 + (bx[ind] - 1.0)**2
    return y


class Rosenbrock(Function):
    def __init__(self, dim_problem, seed=None):
        assert isinstance(dim_problem, int)
        assert isinstance(seed, (type(None), int))
        assert dim_problem > 1

        dim_bx = np.inf
        bounds = np.array([
            [-2.048, 2.048],
        ])
        global_minimizers = np.array([
            [1.0],
        ])
        global_minimum = 0.0
        dim_problem = dim_problem

        function = lambda bx: fun_target(bx, dim_problem)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, dim_problem=dim_problem, seed=seed)



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: January 6, 2023
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx

    first_term = np.sum(bx**2 / 4000.0)

    second_term = 1.0
    for ind in range(1, dim_bx + 1):
        second_term *= np.cos(bx[ind - 1] / np.sqrt(ind))

    y = first_term - second_term + 1.0
    return y


class Griewank(Function):
    def __init__(self, dim_problem, seed=None):
        assert isinstance(dim_problem, int)
        assert isinstance(seed, (type(None), int))

        dim_bx = np.inf
        bounds = np.array([
            [-600.0, 600.0],
        ])
        global_minimizers = np.array([
            [0.0],
        ])
        global_minimum = 0.0
        dim_problem = dim_problem

        function = lambda bx: fun_target(bx, dim_problem)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, dim_problem=dim_problem, seed=seed)



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: December 4, 2022
#

import numpy as np
import pytest

from bayeso_benchmarks.two_dim_kim1 import *

class_fun = Kim1
str_name = 'kim1'

TEST_EPSILON = 1e-5
SCALE_NOISE = 2.0
SEED = 42


def test_init():
    obj_fun = class_fun()

    with pytest.raises(AssertionError) as error:
        class_fun(seed=1.0)
    with pytest.raises(AssertionError) as error:
        class_fun(seed='abc')

def test_validate_properties():
    obj_fun = class_fun()
    obj_fun.validate_properties()

def test_output():
    obj_fun = class_fun()
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [9.91424384],
        [2.97034052],
        [4.2184372],
        [8.54390332],
        [1.6],
        [2.84809668],
        [7.35424384],
        [0.41034052],
        [1.6584372],
    ])

    for elem in obj_fun.output(grids):
        print('{},'.format(elem))

    print(grids)
    print(obj_fun.output(grids))
    print(np.abs(obj_fun.output(grids) - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(obj_fun.output(grids) - truths_grids) < TEST_EPSILON)

def test_output_constant_noise():
    obj_fun = class_fun()
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [11.91424384],
        [4.97034052],
        [6.2184372],
        [10.54390332],
        [3.6],
        [4.84809668],
        [9.35424384],
        [2.41034052],
        [3.6584372],
    ])

    for elem in obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE):
        print('{},'.format(elem))

    print(grids)
    print(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE))
    print(np.abs(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE) - truths_grids) < TEST_EPSILON + SCALE_NOISE)
    assert np.all(np.abs(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE) - truths_grids) < TEST_EPSILON + SCALE_NOISE)

def test_output_gaussian_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [10.90767214],
        [2.69381192],
        [5.51381428],
        [11.58996303],
        [1.13169325],
        [2.37982277],
        [10.51266947],
        [1.94520998],
        [0.71948843],
    ])
    outputs = obj_fun.output_gaussian_noise(grids, scale_noise=SCALE_NOISE)

    for elem in outputs:
        print('{},'.format(elem))

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_name():
    obj_fun = class_fun()
    assert obj_fun.name == str_name

    assert obj_fun.__class__.__name__.lower() == str_name
    assert obj_fun.__class__.__qualname__.lower() == str_name



#
# author: Jungtaek Kim (jungtaek.kim.mail@gmail.com)
# last updated: August 4, 2023
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx

    y = np.sin(bx[0] / 1.0) + np.cos(bx[1] / 1.0) \
        + np.sin(bx[0] / 2.0) + np.cos(bx[1] / 2.0) \
        + np.sin(bx[0] / 4.0) + np.cos(bx[1] / 4.0) \
        + np.sin(bx[0] / 8.0) + np.cos(bx[1] / 8.0) \
        + np.sin(bx[0] / 16.0) + np.cos(bx[1] / 16.0) \
        + 0.0032 * (bx[0] - 20.0)**2 + 0.0016 * (bx[1] - 20.0)**2
    return y


class Kim2(Function):
    def __init__(self, seed=None):
        assert isinstance(seed, (type(None), int))

        dim_bx = 2
        bounds = np.array([
            [-128.0, 128.0],
            [-128.0, 128.0],
        ])
        assert bounds.shape[0] == dim_bx
        assert bounds.shape[1] == 2

        global_minimizers = np.array([
            [-2.1013466, 34.14526252],
        ])
        global_minimum = -3.4543874735
        function = lambda bx: fun_target(bx, dim_bx)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jungtaek.kim.mail@gmail.com)
# last updated: August 4, 2023
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx

    term_1a = (bx[0] + bx[1] + 1.0)**2
    term_1b = 19.0 - 14.0 * bx[0] + 3.0 * bx[0]**2 - 14.0 * bx[1] + 6.0 * bx[0] * bx[1] + 3.0 * bx[1]**2
    term_1 = 1.0 + term_1a * term_1b

    term_2a = (2.0 * bx[0] - 3.0 * bx[1])**2
    term_2b = 18.0 - 32.0 * bx[0] + 12.0 * bx[0]**2 + 48.0 * bx[1] - 36.0 * bx[0] * bx[1] + 27.0 * bx[1]**2
    term_2 = 30.0 + term_2a * term_2b

    y = term_1 * term_2
    return y


class GoldsteinPrice(Function):
    def __init__(self, seed=None):
        assert isinstance(seed, (type(None), int))

        dim_bx = 2
        bounds = np.array([
            [-2.0, 2.0],
            [-2.0, 2.0],
        ])
        assert bounds.shape[0] == dim_bx
        assert bounds.shape[1] == 2

        global_minimizers = np.array([
            [0.0, -1.0],
        ])
        global_minimum = 2.9999999999
        function = lambda bx: fun_target(bx, dim_bx)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jungtaek.kim.mail@gmail.com)
# last updated: August 4, 2023
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx

    first_term = 0.0
    for ind in range(1, 6):
        first_term += ind * np.cos((ind + 1.0) * bx[0] + ind)

    second_term = 0.0
    for ind in range(1, 6):
        second_term += ind * np.cos((ind + 1.0) * bx[1] + ind)

    y = first_term * second_term
    return y


class Shubert(Function):
    def __init__(self, seed=None):
        assert isinstance(seed, (type(None), int))

        dim_bx = 2
        bounds = np.array([
            [-10, 10],
            [-10, 10],
        ])
        assert bounds.shape[0] == dim_bx
        assert bounds.shape[1] == 2

        global_minimizers = np.array([
            [-7.08350641, -7.70831374],
            [5.48286421, -7.70831373],
            [-0.8003211, -7.70831374],
            [4.85805688, -7.08350641],
            [-7.70831374, -0.8003211],
            [-7.70831374, 5.4828642],
            [4.85805688, 5.48286421],
        ])
        global_minimum = -186.7309088311

        function = lambda bx: fun_target(bx, dim_bx)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: December 29, 2022
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx,
):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx

    bw = []
    for x in bx:
        w = 1.0 + (x - 1.0) / 4.0
        bw.append(w)
    bw = np.array(bw)

    y = np.sin(np.pi * bw[0])**2

    for w in bw[:-1]:
        y += (w - 1.0)**2 * (1.0 + 10.0 * np.sin(np.pi * w + 1.0)**2)

    y += (bw[-1] - 1.0)**2 * (1.0 + np.sin(2.0 * np.pi * bw[-1])**2)
    return y


class Levy(Function):
    def __init__(self, dim_problem, seed=None):
        assert isinstance(dim_problem, int)
        assert isinstance(seed, (type(None), int))

        dim_bx = np.inf
        bounds = np.array([
            [-10.0, 10.0],
        ])
        global_minimizers = np.array([
            [1.0],
        ])
        global_minimum = 0.0
        dim_problem = dim_problem

        function = lambda bx: fun_target(bx, dim_problem)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, dim_problem=dim_problem, seed=seed)



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: December 30, 2022
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx

    y = 100.0 * (bx[0]**2 - bx[1])**2
    y += (bx[0] - 1.0)**2
    y += (bx[2] - 1.0)**2
    y += 90.0 * (bx[2]**2 - bx[3])**2
    y += 10.1 * ((bx[1] - 1.0)**2 + (bx[3] - 1.0)**2)
    y += 19.8 * (bx[1] - 1.0) * (bx[3] - 1.0)

    return y


class Colville(Function):
    def __init__(self, seed=None):
        assert isinstance(seed, (type(None), int))

        dim_bx = 4
        bounds = np.array([
            [-10.0, 10.0],
            [-10.0, 10.0],
            [-10.0, 10.0],
            [-10.0, 10.0],
        ])
        assert bounds.shape[0] == dim_bx
        assert bounds.shape[1] == 2

        global_minimizers = np.array([
            [1.0, 1.0, 1.0, 1.0],
        ])
        global_minimum = 0.0
        function = lambda bx: fun_target(bx, dim_bx)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: December 4, 2022
#

import numpy as np
import pytest

from bayeso_benchmarks.inf_dim_rosenbrock import *

class_fun = Rosenbrock
str_name = 'rosenbrock'

TEST_EPSILON = 1e-5


def test_init():
    obj_fun = class_fun(2)

    with pytest.raises(TypeError) as error:
        class_fun()
    with pytest.raises(AssertionError) as error:
        class_fun('abc')
    with pytest.raises(AssertionError) as error:
        class_fun(2.1)
    with pytest.raises(AssertionError) as error:
        class_fun(2, seed='abc')
    with pytest.raises(AssertionError) as error:
        class_fun(2, seed=2.1)

def test_validate_properties():
    obj_fun = class_fun(5)
    obj_fun.validate_properties()

def test_output():
    obj_fun = class_fun(3)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [7.81185245e+03],
        [5.67443514e+03],
        [4.37587862e+03],
        [4.32635663e+03],
        [2.18893931e+03],
        [8.90382790e+02],
        [7.80366045e+03],
        [5.66624314e+03],
        [4.36768662e+03],
        [2.18893931e+03],
        [1.76950891e+03],
        [2.18893931e+03],
        [4.21430400e+02],
        [2.00000000e+00],
        [4.21430400e+02],
        [2.18074731e+03],
        [1.76131691e+03],
        [2.18074731e+03],
        [4.36768662e+03],
        [2.23026930e+03],
        [9.31712780e+02],
        [4.31816463e+03],
        [2.18074731e+03],
        [8.82190790e+02],
        [4.35949462e+03],
        [2.22207730e+03],
        [9.23520780e+02],
    ])
    
    print(grids)
    print(obj_fun.output(grids))
    print(np.abs(obj_fun.output(grids) - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(obj_fun.output(grids) - truths_grids) < TEST_EPSILON)

def test_name():
    obj_fun = class_fun(2)
    assert obj_fun.name == str_name + '_2'

    obj_fun = class_fun(4)
    assert obj_fun.name == str_name + '_4'

    obj_fun = class_fun(16)
    assert obj_fun.name == str_name + '_16'

    assert obj_fun.__class__.__name__.lower() == str_name
    assert obj_fun.__class__.__qualname__.lower() == str_name



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: February 8, 2021
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx, constant):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx
    assert isinstance(constant, float)

    y = constant
    return y


class Constant(Function):
    def __init__(self,
        bounds=np.array([
            [-10.0, 10.0],
        ]),
        constant=0.0,
        seed=None
    ):
        assert isinstance(constant, float)
        assert isinstance(bounds, np.ndarray)
        assert isinstance(seed, (type(None), int))
        assert len(bounds.shape) == 2
        assert bounds.shape[0] == 1
        assert bounds.shape[1] == 2
        assert bounds[0, 0] < bounds[0, 1]

        dim_bx = bounds.shape[0]
        min_bx = bounds[0, 0]
        max_bx = bounds[0, 1]
        global_minimizers = np.array([
            [min_bx],
            [max_bx],
        ])
        global_minimum = constant
        function = lambda bx: fun_target(bx, dim_bx, constant)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jungtaek.kim.mail@gmail.com)
# last updated: August 4, 2023
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx

    y = -1.0 * (bx[1] + 47.0) * np.sin(np.sqrt(np.abs(bx[1] + bx[0] / 2.0 + 47.0))) - bx[0] * np.sin(np.sqrt(np.abs(bx[0] - (bx[1] + 47.0))))
    return y


class Eggholder(Function):
    def __init__(self, seed=None):
        assert isinstance(seed, (type(None), int))

        dim_bx = 2
        bounds = np.array([
            [-512.0, 512.0],
            [-512.0, 512.0],
        ])
        assert bounds.shape[0] == dim_bx
        assert bounds.shape[1] == 2

        global_minimizers = np.array([
            [512.0, 404.2318066],
        ])
        global_minimum = -959.6406627209
        function = lambda bx: fun_target(bx, dim_bx)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jungtaek.kim.mail@gmail.com)
# last updated: August 4, 2023
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx

    y = 0.0

    for ind in range(0, dim_bx):
        y += np.sin(bx[ind]) * np.sin(((ind + 1.0) * bx[ind]**2) / np.pi)**(2.0 * 10.0)
    y *= -1.0

    return y


class Michalewicz(Function):
    def __init__(self, seed=None):
        assert isinstance(seed, (type(None), int))

        dim_bx = 2
        bounds = np.array([
            [0.0, np.pi],
            [0.0, np.pi],
        ])
        assert bounds.shape[0] == dim_bx
        assert bounds.shape[1] == 2

        global_minimizers = np.array([
            [2.20290552, 1.57079632],
        ])
        global_minimum = -1.8013034101
        function = lambda bx: fun_target(bx, dim_bx)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jungtaek.kim.mail@gmail.com)
# last updated: December 13, 2022
#

import numpy as np

EPSILON = 1e-4


class Function:
    def __init__(self, dimensionality, bounds, global_minimizers, global_minimum, function, dim_problem=None, seed=None):
        assert isinstance(dimensionality, int) or dimensionality is np.inf
        assert isinstance(bounds, np.ndarray)
        assert isinstance(global_minimizers, np.ndarray)
        assert isinstance(global_minimum, float)
        assert callable(function)
        assert isinstance(dim_problem, (type(None), int))
        assert isinstance(seed, (type(None), int))
        assert len(bounds.shape) == 2
        assert bounds.shape[1] == 2
        assert (bounds[:, 0] <= bounds[:, 1]).all()

        self._dimensionality = dimensionality
        self._bounds = bounds
        self._global_minimizers = global_minimizers
        self._global_minimum = global_minimum
        self._function = function

        self.dim_problem = dim_problem
        self.random_state = np.random.RandomState(seed)

        self.validate_properties()
        self.set_name()

    @property
    def dimensionality(self):
        return self._dimensionality

    @property
    def bounds(self):
        return self._bounds

    @property
    def global_minimizers(self):
        return self._global_minimizers

    @property
    def global_minimum(self):
        return self._global_minimum

    def set_name(self):
        name = self.__class__.__name__.lower()

        if self.dimensionality is np.inf:
            self.name = f'{name}_{self.dim_problem}'
        else:
            self.name = name

    def get_bounds(self):
        if self.dimensionality is np.inf:
            return np.array(list(self.bounds) * self.dim_problem)
        else:
            return self.bounds

    def get_global_minimizers(self):
        if self.dimensionality is np.inf:
            global_minimizers = self.global_minimizers
            for _ in range(1, self.dim_problem):
                global_minimizers = np.concatenate((global_minimizers, self.global_minimizers), axis=1)

            return global_minimizers
        else:
            return self.global_minimizers

    def function(self, bx):
        if self.dimensionality is np.inf:
            assert self.dim_problem is bx.shape[0]
        else:
            assert self.dimensionality is bx.shape[0]

        return self._function(bx)

    def _output(self, X):
        assert isinstance(X, np.ndarray)

        bounds = self.get_bounds()

        assert np.all(X >= bounds[:, 0])
        assert np.all(X <= bounds[:, 1])

        if len(X.shape) == 2:
            list_results = [self.function(bx) for bx in X]
        else:
            list_results = [self.function(X)]
            
        by = np.array(list_results)
        return by

    def output(self, X):
        by = self._output(X)
        Y = np.expand_dims(by, axis=1)

        assert len(Y.shape) == 2
        assert Y.shape[1] == 1
        return Y

    def output_constant_noise(self, X, scale_noise=0.01):
        assert isinstance(scale_noise, float)

        by = self._output(X)
        by += scale_noise

        Y = np.expand_dims(by, axis=1)

        assert len(Y.shape) == 2
        assert Y.shape[1] == 1
        return Y

    def output_gaussian_noise(self, X, scale_noise=0.01):
        assert isinstance(scale_noise, float)

        by = self._output(X)
        by += scale_noise * self.random_state.randn(by.shape[0])

        Y = np.expand_dims(by, axis=1)

        assert len(Y.shape) == 2
        assert Y.shape[1] == 1
        return Y

    def output_sparse_gaussian_noise(self, X, scale_noise=0.1, sparsity=0.01):
        assert isinstance(scale_noise, float)
        assert isinstance(sparsity, float)
        assert sparsity >= 0.0 and sparsity <= 1.0
        assert sparsity < 0.5

        by = self._output(X)

        if len(X.shape) == 2:
            num_X = X.shape[0]
        else:
            num_X = 1

        noise = self.random_state.randn(num_X)
        mask = self.random_state.uniform(low=0.0, high=1.0, size=num_X) < sparsity
        noise *= mask.astype(float)
        by += scale_noise * noise

        Y = np.expand_dims(by, axis=1)

        assert len(Y.shape) == 2
        assert Y.shape[1] == 1
        return Y

    def output_student_t_noise(self, X, scale_noise=0.01, dof=4.0):
        assert isinstance(scale_noise, float)
        assert isinstance(dof, float)

        by = self._output(X)
        by += scale_noise * self.random_state.standard_t(dof, size=by.shape[0])

        Y = np.expand_dims(by, axis=1)

        assert len(Y.shape) == 2
        assert Y.shape[1] == 1
        return Y

    def output_sparse_student_t_noise(self, X, scale_noise=0.1, dof=4.0, sparsity=0.01):
        assert isinstance(scale_noise, float)
        assert isinstance(dof, float)
        assert isinstance(sparsity, float)
        assert sparsity >= 0.0 and sparsity <= 1.0
        assert sparsity < 0.5

        by = self._output(X)

        if len(X.shape) == 2:
            num_X = X.shape[0]
        else:
            num_X = 1

        noise = self.random_state.standard_t(dof, size=num_X)
        mask = self.random_state.uniform(low=0.0, high=1.0, size=num_X) < sparsity
        noise *= mask.astype(float)
        by += scale_noise * noise

        Y = np.expand_dims(by, axis=1)

        assert len(Y.shape) == 2
        assert Y.shape[1] == 1
        return Y

    def validate_properties(self):
        shape_bounds = self.get_bounds().shape

        global_minimizers = self.get_global_minimizers()
        shape_global_minimizers = global_minimizers.shape

        assert len(shape_bounds) == 2
        assert shape_bounds[1] == 2
        assert len(shape_global_minimizers) == 2
        assert np.all(np.abs(self.output(global_minimizers) - self.global_minimum) < EPSILON)

        if self.dimensionality is np.inf:
            assert shape_bounds[0] == shape_global_minimizers[1]
        else:
            assert self.dimensionality == shape_bounds[0] == shape_global_minimizers[1]

    def sample_grids(self, num_grids):
        assert isinstance(num_grids, int)

        list_grids = []
        for bound in self.get_bounds():
            list_grids.append(np.linspace(bound[0], bound[1], num_grids))
        list_grids_mesh = list(np.meshgrid(*list_grids))
        list_grids = []
        for elem in list_grids_mesh:
            list_grids.append(elem.flatten(order='C'))
        grids = np.vstack(tuple(list_grids))
        grids = grids.T
        return grids

    def sample_uniform(self, num_points, seed=None):
        assert isinstance(num_points, int)
        assert isinstance(seed, (type(None), int))

        random_state_ = np.random.RandomState(seed)

        if self.dimensionality is np.inf:
            dim_problem = self.dim_problem
        else:
            dim_problem = self.dimensionality

        bounds = self.get_bounds()

        points = random_state_.uniform(size=(num_points, dim_problem))
        points = bounds[:, 0] + (bounds[:, 1] - bounds[:, 0]) * points

        return points

    def __call__(self, X):
        return self.output(X)



#
# author: Jungtaek Kim (jungtaek.kim.mail@gmail.com)
# last updated: August 4, 2023
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx

    y = -1.0 * np.abs(np.sin(bx[0]) * np.cos(bx[1]) * np.exp(np.abs(1.0 - np.sqrt(bx[0]**2 + bx[1]**2) / np.pi)))
    return y


class HolderTable(Function):
    def __init__(self, seed=None):
        assert isinstance(seed, (type(None), int))

        dim_bx = 2
        bounds = np.array([
            [-10.0, 10.0],
            [-10.0, 10.0],
        ])
        assert bounds.shape[0] == dim_bx
        assert bounds.shape[1] == 2

        global_minimizers = np.array([
            [8.05502347, 9.66459002],
            [8.05502347, -9.66459002],
            [-8.05502347, 9.66459002],
            [-8.05502347, -9.66459002],
        ])
        global_minimum = -19.2085025679
        function = lambda bx: fun_target(bx, dim_bx)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: January 3, 2023
#

import numpy as np
import scipy.optimize as scio
import pytest

from bayeso_benchmarks import Ackley
from bayeso_benchmarks import Branin
from bayeso_benchmarks import GramacyAndLee2012
from bayeso_benchmarks import HolderTable
from bayeso_benchmarks import Kim1
from bayeso_benchmarks import Kim2
from bayeso_benchmarks import Kim3
from bayeso_benchmarks import Rastrigin
from bayeso_benchmarks import Michalewicz
from bayeso_benchmarks import Shubert


TEST_EPSILON = 1e-7


def _test_global_minimum(obj_fun):
    fun_target = lambda bx: np.squeeze(obj_fun.output(bx), axis=1)

    grids = obj_fun.sample_grids(100)

    list_bx = []
    list_by = []

    for initial in grids:
        results = scio.minimize(fun_target, initial, method='L-BFGS-B', bounds=obj_fun.get_bounds())

        list_bx.append(results.x)
        list_by.append(results.fun)

    ind_minimum = np.argmin(np.squeeze(list_by))
    bx_best = list_bx[ind_minimum]
    y_best = list_by[ind_minimum]

    print(bx_best)
    print(obj_fun.global_minimum)
    print(y_best)

    X = np.array(list_bx)
    by = np.squeeze(list_by)
    indices = np.argsort(by)
    X = X[indices]
    by = by[indices]

    for bx_candidate, y_candidate in zip(X, by):
        if np.abs(obj_fun.global_minimum - y_candidate) < 1e0:
            print(bx_candidate, y_candidate)

    assert np.abs(obj_fun.global_minimum - y_best) < TEST_EPSILON

    for global_minimizer in obj_fun.get_global_minimizers():
        assert np.abs(obj_fun.global_minimum - fun_target(global_minimizer)[0]) < TEST_EPSILON

def test_global_minimum_branin():
    class_fun = Branin
    obj_fun = class_fun()

    _test_global_minimum(obj_fun)

def test_global_minimum_gramacyandlee2012():
    class_fun = GramacyAndLee2012
    obj_fun = class_fun()

    _test_global_minimum(obj_fun)

def test_global_minimum_holdertable():
    class_fun = HolderTable
    obj_fun = class_fun()

    _test_global_minimum(obj_fun)

def test_global_minimum_kim1():
    class_fun = Kim1
    obj_fun = class_fun()

    _test_global_minimum(obj_fun)

def test_global_minimum_kim2():
    class_fun = Kim2
    obj_fun = class_fun()

    _test_global_minimum(obj_fun)

def test_global_minimum_kim3():
    class_fun = Kim3
    obj_fun = class_fun()

    _test_global_minimum(obj_fun)

def test_global_minimum_shubert():
    class_fun = Shubert
    obj_fun = class_fun()

    _test_global_minimum(obj_fun)



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: February 8, 2021
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx, slope):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx
    assert isinstance(slope, float)

    y = slope * bx[0]
    return y


class Linear(Function):
    def __init__(self,
        bounds=np.array([
            [-10, 10],
        ]),
        slope=1.0,
        seed=None
    ):
        assert isinstance(slope, float)
        assert isinstance(bounds, np.ndarray)
        assert isinstance(seed, (type(None), int))
        assert len(bounds.shape) == 2
        assert bounds.shape[0] == 1
        assert bounds.shape[1] == 2
        assert bounds[0, 0] < bounds[0, 1]

        dim_bx = bounds.shape[0]

        if slope > 0.0:
            global_minimizers = np.array([
                [bounds[0, 0]],
            ])
            global_minimum = slope * bounds[0, 0]
        else:
            global_minimizers = np.array([
                [bounds[0, 1]],
            ])
            global_minimum = slope * bounds[0, 1]
        function = lambda bx: fun_target(bx, dim_bx, slope)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: December 4, 2022
#

import numpy as np
import pytest

from bayeso_benchmarks.two_dim_kim3 import *

class_fun = Kim3
str_name = 'kim3'

TEST_EPSILON = 1e-5
SCALE_NOISE = 2.0
SEED = 42


def test_init():
    obj_fun = class_fun()

    with pytest.raises(AssertionError) as error:
        class_fun(seed=1.0)
    with pytest.raises(AssertionError) as error:
        class_fun(seed='abc')

def test_validate_properties():
    obj_fun = class_fun()
    obj_fun.validate_properties()

def test_output():
    obj_fun = class_fun()
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [208.90875507],
        [72.1885316],
        [145.18351361],
        [145.56021506],
        [8.83995458],
        [81.83497908],
        [176.14075507],
        [39.4205316],
        [112.41551361],
    ])

    for elem in obj_fun.output(grids):
        print('{},'.format(elem))

    print(grids)
    print(obj_fun.output(grids))
    print(np.abs(obj_fun.output(grids) - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(obj_fun.output(grids) - truths_grids) < TEST_EPSILON)

def test_output_constant_noise():
    obj_fun = class_fun()
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [210.90875507],
        [74.1885316],
        [147.18351361],
        [147.56021506],
        [10.83995458],
        [83.83497908],
        [178.14075507],
        [41.4205316],
        [114.41551361],
    ])

    for elem in obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE):
        print('{},'.format(elem))

    print(grids)
    print(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE))
    print(np.abs(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE) - truths_grids) < TEST_EPSILON + SCALE_NOISE)
    assert np.all(np.abs(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE) - truths_grids) < TEST_EPSILON + SCALE_NOISE)

def test_output_gaussian_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [209.90218338],
        [71.912003],
        [146.47889068],
        [148.60627477],
        [8.37164783],
        [81.36670517],
        [179.2991807],
        [40.95540106],
        [111.47656484],
    ])
    outputs = obj_fun.output_gaussian_noise(grids, scale_noise=SCALE_NOISE)

    for elem in outputs:
        print('{},'.format(elem))

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_name():
    obj_fun = class_fun()
    assert obj_fun.name == str_name

    assert obj_fun.__class__.__name__.lower() == str_name
    assert obj_fun.__class__.__qualname__.lower() == str_name



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: January 27, 2023
#


STR_VERSION = '0.2.0'


def test_version_bayeso():
    import bayeso_benchmarks
    assert bayeso_benchmarks.__version__ == STR_VERSION

def test_version_setup():
    import pkg_resources
    assert pkg_resources.require('bayeso-benchmarks')[0].version == STR_VERSION



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: January 3, 2023
#

import numpy as np
import pytest

from bayeso_benchmarks.two_dim_easom import *

class_fun = Easom
str_name = 'easom'

TEST_EPSILON = 1e-5
SCALE_NOISE = 2.0
SEED = 42


def test_init():
    obj_fun = class_fun()

    with pytest.raises(AssertionError) as error:
        class_fun(seed=1.0)
    with pytest.raises(AssertionError) as error:
        class_fun(seed='abc')

def test_validate_properties():
    obj_fun = class_fun()
    obj_fun.validate_properties()

def test_output():
    obj_fun = class_fun()
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [0.0],
        [0.0],
        [0.0],
        [0.0],
        [-2.67528799e-09],
        [0.0],
        [0.0],
        [0.0],
        [0.0],
    ])

    print(grids)
    print(obj_fun.output(grids))
    print(np.abs(obj_fun.output(grids) - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(obj_fun.output(grids) - truths_grids) < TEST_EPSILON)

def test_call():
    obj_fun = class_fun()
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [0.0],
        [0.0],
        [0.0],
        [0.0],
        [-2.67528799e-09],
        [0.0],
        [0.0],
        [0.0],
        [0.0],
    ])

    print(grids)
    print(obj_fun(grids))
    print(np.abs(obj_fun(grids) - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(obj_fun(grids) - truths_grids) < TEST_EPSILON)

def test_output_constant_noise():
    obj_fun = class_fun()
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [2.0],
        [2.0],
        [2.0],
        [2.0],
        [2.0],
        [2.0],
        [2.0],
        [2.0],
        [2.0],
    ])

    print(grids)
    print(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE))
    print(np.abs(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE) - truths_grids) < TEST_EPSILON + SCALE_NOISE)
    assert np.all(np.abs(obj_fun.output_constant_noise(grids, scale_noise=SCALE_NOISE) - truths_grids) < TEST_EPSILON + SCALE_NOISE)

def test_output_gaussian_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [0.99342831],
        [-0.2765286],
        [1.29537708],
        [3.04605971],
        [-0.46830675],
        [-0.46827391],
        [3.15842563],
        [1.53486946],
        [-0.93894877],
    ])
    outputs = obj_fun.output_gaussian_noise(grids, scale_noise=SCALE_NOISE)

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_output_sparse_gaussian_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [9.93428306e-01],
        [-2.76528602e-01],
        [0.0],
        [0.0],
        [-2.67528799e-09],
        [-4.68273914e-01],
        [0.0],
        [1.53486946],
        [-9.38948772e-01],
    ])
    outputs = obj_fun.output_sparse_gaussian_noise(grids, scale_noise=SCALE_NOISE, sparsity=0.3)

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_output_student_t_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [1.1492346],
        [-2.16338047],
        [2.63811703],
        [-1.58355153],
        [1.4728255],
        [3.78670737],
        [-3.92829244],
        [-1.11199715],
        [-1.14321992],
    ])
    outputs = obj_fun.output_student_t_noise(grids, scale_noise=SCALE_NOISE, dof=4.0)

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_output_sparse_student_t_noise():
    obj_fun = class_fun(seed=SEED)
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [1.14923460],
        [-2.16338047],
        [0.0],
        [0.0],
        [-2.67528799e-09],
        [0.0],
        [-3.92829244],
        [0.0],
        [0.0],
    ])
    outputs = obj_fun.output_sparse_student_t_noise(grids, scale_noise=SCALE_NOISE, dof=4.0, sparsity=0.3)

    print(grids)
    print(outputs)
    print(np.abs(outputs - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(outputs - truths_grids) < TEST_EPSILON)

def test_name():
    obj_fun = class_fun()
    assert obj_fun.name == str_name

    assert obj_fun.__class__.__name__.lower() == str_name
    assert obj_fun.__class__.__qualname__.lower() == str_name



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: December 4, 2022
#

import numpy as np
import pytest

from bayeso_benchmarks.two_dim_holdertable import *

class_fun = HolderTable
str_name = 'holdertable'

TEST_EPSILON = 1e-5


def test_init():
    obj_fun = class_fun()

    with pytest.raises(AssertionError) as error:
        class_fun(seed='abc')
    with pytest.raises(AssertionError) as error:
        class_fun(seed=2.1)

def test_validate_properties():
    obj_fun = class_fun()
    obj_fun.validate_properties()

def test_output():
    obj_fun = class_fun()
    bounds = obj_fun.get_bounds()

    grids = obj_fun.sample_grids(3)
    truths_grids = np.array([
        [-15.14022386],
        [0.],
        [-15.14022386],
        [-4.827514],
        [-0.],
        [-4.827514],
        [-15.14022386],
        [0.],
        [-15.14022386],
    ])
    
    print(grids)
    print(obj_fun.output(grids))
    print(np.abs(obj_fun.output(grids) - truths_grids) < TEST_EPSILON)
    assert np.all(np.abs(obj_fun.output(grids) - truths_grids) < TEST_EPSILON)

def test_name():
    obj_fun = class_fun()
    assert obj_fun.name == str_name

    assert obj_fun.__class__.__name__.lower() == str_name
    assert obj_fun.__class__.__qualname__.lower() == str_name



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: February 8, 2021
#

import numpy as np

from bayeso_benchmarks.benchmark_base import Function


def fun_target(bx, dim_bx):
    assert len(bx.shape) == 1
    assert bx.shape[0] == dim_bx

    y = 2 * bx[0]**2 - 1.05 * bx[0]**4 + 1.0 / 6.0 * bx[0]**6 + bx[0] * bx[1] + bx[1]**2
    return y


class ThreeHumpCamel(Function):
    def __init__(self, seed=None):
        assert isinstance(seed, (type(None), int))

        dim_bx = 2
        bounds = np.array([
            [-5.0, 5.0],
            [-5.0, 5.0],
        ])
        assert bounds.shape[0] == dim_bx
        assert bounds.shape[1] == 2

        global_minimizers = np.array([
            [0.0, 0.0],
        ])
        global_minimum = 0.0
        function = lambda bx: fun_target(bx, dim_bx)

        super().__init__(dim_bx, bounds, global_minimizers, global_minimum, function, seed=seed)



#
# author: Jungtaek Kim (jtkim@postech.ac.kr)
# last updated: January 4, 2023
#


def test_import_benchmarks():
    import bayeso_benchmarks

def test_all_benchmarks():
    import bayeso_benchmarks

    list_str_names = [
        'ackley',
        'cosines',
        'levy',
        'rastrigin',
        'rosenbrock',
        'sphere',
        'zakharov',
        'constant',
        'gramacyandlee2012',
        'linear',
        'step',
        'colville',
        'hartmann3d',
        'hartmann6d',
        'beale',
        'bohachevsky',
        'branin',
        'bukin6',
        'dejong5',
        'dropwave',
        'easom',
        'eggholder',
        'goldsteinprice',
        'holdertable',
        'kim1',
        'kim2',
        'kim3',
        'michalewicz',
        'shubert',
        'sixhumpcamel',
        'threehumpcamel',
    ]

    names = []
    for class_benchmark in bayeso_benchmarks.all_benchmarks:
        print(class_benchmark.__name__.lower())
        names.append(class_benchmark.__name__.lower())

    for str_name in list_str_names:
        assert str_name in names

    qualnames = []
    for class_benchmark in bayeso_benchmarks.all_benchmarks:
        print(class_benchmark.__qualname__.lower())
        qualnames.append(class_benchmark.__qualname__.lower())

    for str_name in list_str_names:
        assert str_name in qualnames

def test_num_benchmarks():
    import bayeso_benchmarks

    assert bayeso_benchmarks.num_benchmarks == 32

def test_import_benchmark_base():
    import bayeso_benchmarks.benchmark_base
    from bayeso_benchmarks.benchmark_base import Function

def test_import_one_dim_constant():
    import bayeso_benchmarks.one_dim_constant
    from bayeso_benchmarks.one_dim_constant import Constant
    from bayeso_benchmarks import Constant

def test_import_one_dim_gramacyandlee2012():
    import bayeso_benchmarks.one_dim_gramacyandlee2012
    from bayeso_benchmarks.one_dim_gramacyandlee2012 import GramacyAndLee2012
    from bayeso_benchmarks import GramacyAndLee2012

def test_import_one_dim_linear():
    import bayeso_benchmarks.one_dim_linear
    from bayeso_benchmarks.one_dim_linear import Linear
    from bayeso_benchmarks import Linear

def test_import_one_dim_step():
    import bayeso_benchmarks.one_dim_step
    from bayeso_benchmarks.one_dim_step import Step
    from bayeso_benchmarks import Step

def test_import_two_dim_beale():
    import bayeso_benchmarks.two_dim_beale
    from bayeso_benchmarks.two_dim_beale import Beale
    from bayeso_benchmarks import Beale

def test_import_two_dim_bohachevsky():
    import bayeso_benchmarks.two_dim_bohachevsky
    from bayeso_benchmarks.two_dim_bohachevsky import Bohachevsky
    from bayeso_benchmarks import Bohachevsky

def test_import_two_dim_branin():
    import bayeso_benchmarks.two_dim_branin
    from bayeso_benchmarks.two_dim_branin import Branin
    from bayeso_benchmarks import Branin

def test_import_two_dim_bukin6():
    import bayeso_benchmarks.two_dim_bukin6
    from bayeso_benchmarks.two_dim_bukin6 import Bukin6
    from bayeso_benchmarks import Bukin6

def test_import_two_dim_dropwave():
    import bayeso_benchmarks.two_dim_dropwave
    from bayeso_benchmarks.two_dim_dropwave import DropWave
    from bayeso_benchmarks import DropWave

def test_import_two_dim_easom():
    import bayeso_benchmarks.two_dim_easom
    from bayeso_benchmarks.two_dim_easom import Easom
    from bayeso_benchmarks import Easom

def test_import_two_dim_eggholder():
    import bayeso_benchmarks.two_dim_eggholder
    from bayeso_benchmarks.two_dim_eggholder import Eggholder
    from bayeso_benchmarks import Eggholder

def test_import_two_dim_goldsteinprice():
    import bayeso_benchmarks.two_dim_goldsteinprice
    from bayeso_benchmarks.two_dim_goldsteinprice import GoldsteinPrice
    from bayeso_benchmarks import GoldsteinPrice

def test_import_two_dim_holdertable():
    import bayeso_benchmarks.two_dim_holdertable
    from bayeso_benchmarks.two_dim_holdertable import HolderTable
    from bayeso_benchmarks import HolderTable

def test_import_two_dim_kim1():
    import bayeso_benchmarks.two_dim_kim1
    from bayeso_benchmarks.two_dim_kim1 import Kim1
    from bayeso_benchmarks import Kim1

def test_import_two_dim_kim2():
    import bayeso_benchmarks.two_dim_kim2
    from bayeso_benchmarks.two_dim_kim2 import Kim2
    from bayeso_benchmarks import Kim2

def test_import_two_dim_kim3():
    import bayeso_benchmarks.two_dim_kim3
    from bayeso_benchmarks.two_dim_kim3 import Kim3
    from bayeso_benchmarks import Kim3

def test_import_two_dim_michalewicz():
    import bayeso_benchmarks.two_dim_michalewicz
    from bayeso_benchmarks.two_dim_michalewicz import Michalewicz
    from bayeso_benchmarks import Michalewicz

def test_import_two_dim_shubert():
    import bayeso_benchmarks.two_dim_shubert
    from bayeso_benchmarks.two_dim_shubert import Shubert
    from bayeso_benchmarks import Shubert

def test_import_two_dim_sixhumpcamel():
    import bayeso_benchmarks.two_dim_sixhumpcamel
    from bayeso_benchmarks.two_dim_sixhumpcamel import SixHumpCamel
    from bayeso_benchmarks import SixHumpCamel

def test_import_two_dim_threehumpcamel():
    import bayeso_benchmarks.two_dim_threehumpcamel
    from bayeso_benchmarks.two_dim_threehumpcamel import ThreeHumpCamel
    from bayeso_benchmarks import ThreeHumpCamel

def test_import_four_dim_colville():
    import bayeso_benchmarks.four_dim_colville
    from bayeso_benchmarks.four_dim_colville import Colville
    from bayeso_benchmarks import Colville

def test_import_three_dim_hartmann3d():
    import bayeso_benchmarks.three_dim_hartmann3d
    from bayeso_benchmarks.three_dim_hartmann3d import Hartmann3D
    from bayeso_benchmarks import Hartmann3D

def test_import_six_dim_hartmann6d():
    import bayeso_benchmarks.six_dim_hartmann6d
    from bayeso_benchmarks.six_dim_hartmann6d import Hartmann6D
    from bayeso_benchmarks import Hartmann6D

def test_import_inf_dim_ackley():
    import bayeso_benchmarks.inf_dim_ackley
    from bayeso_benchmarks.inf_dim_ackley import Ackley
    from bayeso_benchmarks import Ackley

def test_import_inf_dim_cosines():
    import bayeso_benchmarks.inf_dim_cosines
    from bayeso_benchmarks.inf_dim_cosines import Cosines
    from bayeso_benchmarks import Cosines

def test_import_inf_dim_griewank():
    import bayeso_benchmarks.inf_dim_griewank
    from bayeso_benchmarks.inf_dim_griewank import Griewank
    from bayeso_benchmarks import Griewank

def test_import_inf_dim_levy():
    import bayeso_benchmarks.inf_dim_levy
    from bayeso_benchmarks.inf_dim_levy import Levy
    from bayeso_benchmarks import Levy

def test_import_inf_dim_rastrigin():
    import bayeso_benchmarks.inf_dim_rastrigin
    from bayeso_benchmarks.inf_dim_rastrigin import Rastrigin
    from bayeso_benchmarks import Rastrigin

def test_import_inf_dim_rosenbrock():
    import bayeso_benchmarks.inf_dim_rosenbrock
    from bayeso_benchmarks.inf_dim_rosenbrock import Rosenbrock
    from bayeso_benchmarks import Rosenbrock

def test_import_inf_dim_sphere():
    import bayeso_benchmarks.inf_dim_sphere
    from bayeso_benchmarks.inf_dim_sphere import Sphere
    from bayeso_benchmarks import Sphere

def test_import_inf_dim_zakharov():
    import bayeso_benchmarks.inf_dim_zakharov
    from bayeso_benchmarks.inf_dim_zakharov import Zakharov
    from bayeso_benchmarks import Zakharov
